/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.lidar.amapvox.engine3d.resources.textures;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.util.texture.TextureData;
import com.jogamp.opengl.util.texture.TextureIO;
import java.io.IOException;
import java.nio.Buffer;
import net.jcip.annotations.ThreadSafe;

@ThreadSafe
public class Texture2D extends Texture {
    
    private final Buffer image;

    public Texture2D(final String identifier, final int width, final int height,
            final TextureFormat format, final boolean generateMipmaps) {
        super(identifier, width, height, format, generateMipmaps);
        this.image = null;
    }
    
    public Texture2D(final String identifier, final int width, final int height,
            final TextureFormat format, final boolean generateMipmaps, 
            final String path) throws IOException {
        super(identifier, width, height, format, generateMipmaps);
        
        final TextureData data = TextureIO.newTextureData(
                GLProfile.getGL2GL3(), 
                getClass().getResource(path), 
                getInternalFormat(), getExternalFormat(),
                false, path.split("\\.")[1]);
        image = data.getBuffer();
    }
    
    @Override
    protected void onLoad(final GL gl) {
        super.onLoad(gl);
                        
        gl.glBindTexture(GL.GL_TEXTURE_2D, getIndex());
        
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, 
                GL.GL_CLAMP_TO_EDGE);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, 
                GL.GL_CLAMP_TO_EDGE);
        
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, 
                GL.GL_LINEAR);
        
        if (isGenerateMipmaps()) {
            gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, 
                    GL.GL_LINEAR_MIPMAP_LINEAR);
        } else {
            gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, 
                    GL.GL_LINEAR);
        }
        
        gl.glTexImage2D(GL.GL_TEXTURE_2D, 0, getInternalFormat(), getWidth(),
                getHeight(), 0, getExternalFormat(), GL.GL_UNSIGNED_BYTE, image);
        
        if (isGenerateMipmaps()) {
            gl.glGenerateMipmap(GL.GL_TEXTURE_2D);
        }
    }
    
    @Override
    protected void onBind(final GL gl) {
        gl.glBindTexture(GL.GL_TEXTURE_2D, getIndex());
    }

    @Override
    protected void onUnbind(final GL gl) {
        gl.glBindTexture(GL.GL_TEXTURE_2D, 0);
    }
    
}
