/* Uniform area */
uniform sampler2D screenTexture;

/* Output color */
out vec4 color;

/**
 * Apply a sharpen effect to the specified texture.
 */
vec4 sharpen(in sampler2D tex, in vec2 coords, in float offset)
{
    float dx = offset / resolution.x;
    float dy = offset / resolution.y;
    vec4 sum = vec4(0.0);
    sum += -0.5 * texture2D(tex, coords + vec2( -1.0 * dx , 0.0 * dy));
    sum += -0.5 * texture2D(tex, coords + vec2( 0.0 * dx , -1.0 * dy));
    sum +=  3.0 * texture2D(tex, coords + vec2( 0.0 * dx , 0.0 * dy));
    sum += -0.5 * texture2D(tex, coords + vec2( 0.0 * dx , 1.0 * dy));
    sum += -0.5 * texture2D(tex, coords + vec2( 1.0 * dx , 0.0 * dy));
    return sum;
}

void main(void)
{
    vec3 screenColor = sharpen(screenTexture, viewSpaceTexcoord, 1).rgb;
    float alpha = texture(screenTexture, viewSpaceTexcoord).a;
    color = vec4(screenColor, alpha);
}